package ${basePackage}.module.system.mgr;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.frame.auth.Token;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.utils.IDgenerator;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.utils.Message;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.VisitorRes;

/**
 * VISITOR_RES - 应用资源关系
 *
 * @author wangbing
 * @version 0.0.1
 * @since 2020-06-25
 */
@Transactional
@Service
public class VisitorResManagerImpl implements VisitorResManager {

    @Autowired
    private ${basePackage}.module.system.mpr.VisitorResMapper visitorResMapper;

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public ${basePackage}.module.system.rsp.VisitorResCreateResponse create(${basePackage}.module.system.req.VisitorResCreateRequest request, Token token) {
        ${basePackage}.module.system.rsp.VisitorResCreateResponse response = new ${basePackage}.module.system.rsp.VisitorResCreateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long id = IDgenerator.nextId();
        VisitorRes entity = MapperUtil.map(request, VisitorRes.class);
        entity.setId(id);

        long result = visitorResMapper.insert(entity, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.CREATE_FAILURE);
            return response;
        }
        response.setId(id);

        return response;
    }

    /**
     * 逻辑删除
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public ${basePackage}.module.system.rsp.VisitorResDeleteResponse delete(${basePackage}.module.system.req.VisitorResDeleteRequest request, Token token) {
        ${basePackage}.module.system.rsp.VisitorResDeleteResponse response = new ${basePackage}.module.system.rsp.VisitorResDeleteResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = visitorResMapper.delete(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.DELETE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public ${basePackage}.module.system.rsp.VisitorResUpdateResponse update(${basePackage}.module.system.req.VisitorResUpdateRequest request, Token token) {
        ${basePackage}.module.system.rsp.VisitorResUpdateResponse response = new ${basePackage}.module.system.rsp.VisitorResUpdateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = visitorResMapper.update(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.UPDATE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public ${basePackage}.module.system.rsp.VisitorResFindResponse find(${basePackage}.module.system.req.VisitorResFindRequest request, Token token) {
        ${basePackage}.module.system.rsp.VisitorResFindResponse response = new ${basePackage}.module.system.rsp.VisitorResFindResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        if (request.getPageSize() != 0) {
            PageHelper.startPage(request.getPageNumber(), request.getPageSize());
        }
        if (StringUtil.isNotEmpty(request.getSortKey())) {
            PageHelper.orderBy(request.getSortKey() + " " + request.getSortType());
        }
        PageInfo<VisitorRes> pageInfo = new PageInfo<>(visitorResMapper.find(request, token));

        response.setResult(pageInfo.getList());
        response.setTotalCount(pageInfo.getTotal());

        return response;
    }
}
